package App::Nighttime::Client;

use feature 'state';
use strict;
use warnings;

use parent 'App::Nighttime';

use App::Nighttime::DateTime;
use DateTime;
use LWP::UserAgent;
use Sys::Hostname;
use Sys::Utmp;
use Text::Glob qw(match_glob);

sub _user_logged_in {
  my $utmp = new Sys::Utmp;

  while (my $utent = $utmp->getutent()) {
    return 1 if $utent->user_process;
  }

  return 0;
}

sub _low_uptime($$) {
  my ($self, $schedule) = @_;
  my $c = $self->config;

  my $min_uptime = $schedule->{min_uptime} // $c->{min_uptime};
  return 0 unless defined $min_uptime;

  open my $fh, "<", "/proc/uptime"
    or $self->croak("Could not open '/proc/uptime': $!");
  my ($uptime, undef) = split /\s+/, <$fh>, 2;
  close $fh;

  return $uptime < $min_uptime;
}

sub _high_load($) {
  my ($self, $schedule) = @_;
  my $c = $self->config;

  my $max_load = $schedule->{max_load} // $c->{max_load};
  return 0 unless defined $max_load;

  open my $fh, "<", "/proc/loadavg"
    or $self->croak("Could not open '/proc/loadavg': $!");
  my ($load, undef) = split /\s+/, <$fh>, 2;
  close $fh;

  return $load > $max_load;
}

sub _in_use($$) {
  my ($self, $schedule) = @_;

  $self->debug("in_use: user logged in?");
  return 1 if $self->_user_logged_in;
  $self->debug("in_use: uptime too low?");
  return 1 if $self->_low_uptime($schedule);
  $self->debug("in_use: load too high?");
  return 1 if $self->_high_load($schedule);

  $self->debug("in_use: all tests passed.");
  return 0;
}

sub _shutdown {
  my ($self)   = @_;
  my $c        = $self->config;
  my $shutdown = $c->{shutdown} // "/sbin/shutdown -hP +10 This computer is being shut down by Nighttime &> /dev/null";

  my $r = $self->_system($shutdown);
  if (defined $r) {
    $self->croak("Initiating shutdown failed: $r");
  }
}

sub _schedule_wakeup {
  my ($self, $time) = @_;

  my $url = $self->config->{url}
    or $self->croak("No URL for server found.");

  my $ua = LWP::UserAgent->new;
  $ua->timeout(10);
  $ua->agent("Nighttime/" . $self->version . " " . $ua->_agent);

  my $r = $ua->post($url, { time => $time->epoch });
  unless ($r->is_success) {
    $self->carp("Requesting wake-up failed: " . $r->status_line);
  }
  return $r->is_success;
}

sub _match_hostname($) {
  my ($expr) = @_;
  state $hostname = hostname;

  if ($expr =~ m,^/(.*)/$,) {
    return $hostname =~ $1;
  }
  $expr = "$expr.*" unless $expr =~ /\.|\*$/;
  return scalar match_glob($expr, $hostname);
}

sub _current_schedule {
  my ($self, $now) = @_;
  my $c = $self->config;
  my $schedules = $c->{schedule}
    or $self->croak("No schedules found.");
  $schedules = [$schedules] unless ref($schedules) eq "ARRAY";

  for my $schedule (@$schedules) {
    next if defined $schedule->{host} && !_match_hostname($schedule->{host});
    if (App::Nighttime::DateTime::matches($now, $schedule->{shutdown})) {
      return $schedule;
    }
  }
  return undef;
}

sub run {
  my ($self) = @_;
  my $now = DateTime->now->set_time_zone("local");

  my $schedule = $self->_current_schedule($now);

  return unless defined $schedule;
  return if $self->_in_use($schedule);

  my $wakeup = $schedule->{wakeup} // $self->config->{wakeup};
  if ($wakeup) {
    my $time = App::Nighttime::DateTime::parse_time_expression($now, $wakeup);
    $self->_schedule_wakeup($time);
  }
  else {
    $self->debug("No wakeup requested.");
  }

  $self->_shutdown;
}

1;
# vim: set et sw=2:
