package App::Nighttime::Cron;

use strict;
use warnings;

use parent 'App::Nighttime';

use App::Nighttime::HostInfo;
use File::Glob ':glob';

sub _read_db($) {
  my ($self) = @_;

  my @db;

  my $fname = $self->config->{db} . "/db";
  open my $fh, "<", $fname
    or $self->croak("Could not open database '$fname' for reading: $!");

  while (<$fh>) {
    my ($time, $ip) = split / /, $_, 2;
    next unless $time && $ip;
    chomp $ip;
    push @db, [ $time, $ip ];
  }

  return \@db;
}

sub _read_new($) {
  my ($self) = @_;
  my $fname = $self->config->{db} . "/new/new-*";

  my @files = bsd_glob($fname);
  my @db;

  for my $file (@files) {
    open my $fh, "<", $file or next;
    my $time = <$fh>; chomp $time;
    my $ip   = <$fh>; chomp $ip;
    close $fh;
    unlink $file;

    next unless $time =~ /^[0-9]+$/;
    next unless $ip   =~ /^[0-9.:]+$/;

    push @db, [ $time, $ip ];
  }

  return \@db;
}

sub _sort($$) {
  my ($self, $db) = @_;
  my @db = sort { $$a[0] <=> $$b[0] } @$db;
  return \@db;
}

sub _write_db($$) {
  my ($self, $db) = @_;
  my $fname = $self->config->{db} . "/db";

  open my $fh, ">", $fname
    or $self->croak("Could not open database '$fname' for writing: $!");

  for my $entry (@$db) {
    print $fh $$entry[0] . " " . $$entry[1] . "\n";
  }
}

sub _wakeup($$) {
  my ($self, $ip) = @_;
  my $c = $self->config;

  my $wakeonlan = $c->{wakeonlan} // "wakeonlan";

  my $mac = $self->_hostinfo->ip_to_mac($ip);
  unless (defined $mac) {
    $self->carp("MAC address for IP $ip not found.");
    return 0;
  }
  unless ($mac =~ /^[0-9A-Fa-f:]+$/) {
    $self->carp("Does not look like a MAC address: '$mac' (IP was '$ip')");
    return 0;
  }

  my $mesg = $self->_system($wakeonlan, $mac);

  my $sleep = $c->{sleep};
  sleep($sleep) if defined $sleep;

  if (defined $mesg) {
    $self->carp("Running wakeonlan for IP $ip failed: $mesg");
    return 0;
  }
  return 1;
}

sub _hostinfo($) {
  my ($self) = @_;
  return $self->{hostinfo} if defined $self->{hostinfo};

  my $hi = $self->config->{hostinfo}
    or $self->croak("No hostinfo section in configuration file.");
  my ($type, $options) = each %{ $hi };
  #my $hostinfo = App::Nighttime::HostInfo->new($type, $options);
  use App::Nighttime::HostInfo::LDAP;
  my $hostinfo = App::Nighttime::HostInfo::LDAP->new($options);

  $self->{hostinfo} = $hostinfo;
  return $self->{hostinfo};
}

sub run($) {
  my ($self) = @_;
  my $now = time;

  my $db = $self->_read_db;
  push @$db, @{ $self->_read_new };
  $db = $self->_sort($db);

  my $i;
  for ($i = 0; $i < @$db; $i++) {
    my ($time, $ip) = @{ $$db[$i] };
    last if ($time > $now);
    delete $$db[$i];
    $self->_wakeup($ip);
  }

  $self->_write_db($db);
}

1;
# vim: set et sw=2:
