package App::Nighttime::DateTime;

use strict;
use warnings;

use Carp;

sub _parse_time($) {
  my $time = shift;
  my $l    = length($time);

  my ($hour, $minute) = split /\./, $time, 2;
  $minute = 0 unless defined $minute;

  croak "Could not understand time: '$time'" unless defined $hour;

  return ($hour, $minute);
}

sub _today($) {
  my ($now) = @_;
  return $now->truncate(to => "day");
}

sub _tomorrow($) {
  my ($now) = @_;
  return $now->add(days => 1)->truncate(to => "day");
}

sub _next_workday($) {
  my ($now) = @_;

  my $dow = $now->day_of_week;

  # How many days to the next weekday
  my $days = $dow == 5 ? 3
           : $dow == 6 ? 2
           :             1;

  return $now->add(days => $days)->truncate(to => "day");
}

sub parse_time_expression($$) {
  my ($now, $expr) = @_;

  my ($day_expr, $time_expr) = $expr =~ /^\s*(.+\S)\s+at\s+(\S+)\s*$/;
  $day_expr =~ s/\s+/ /;

  my %DAY_EXPR = (
    "next workday" => \&_next_workday,
    "today"        => \&_today,
    "tomorrow"     => \&_tomorrow,
  );

  my $day_func = $DAY_EXPR{$day_expr};
  croak "Unknown expression for day: '$day_expr'" unless defined $day_func;

  my $time = &$day_func($now);
  my ($hour, $minute) = _parse_time($time_expr);
  $time->set_hour($hour)->set_minute($minute);

  return $time;
}

sub _parse_days($) {
  my ($days_s) = @_;

  my %DAYS = (
    "monday"    => 0x01, "mon" => 0x01, "mo" => 0x01,
    "tuesday"   => 0x02, "tue" => 0x02, "tu" => 0x02,
    "wednesday" => 0x04, "wed" => 0x04, "we" => 0x04,
    "thursday"  => 0x08, "thu" => 0x08, "th" => 0x08,
    "friday"    => 0x10, "fri" => 0x10, "fr" => 0x10,
    "saturday"  => 0x20, "sat" => 0x20, "sa" => 0x20,
    "sunday"    => 0x40, "sun" => 0x40, "su" => 0x40,
    "weekend"   => 0x60,
    "workday"   => 0x3f,
    "all"       => 0x7f, "*"   => 0x7f,
  );

  $days_s =~ s/\s*//g;
  $days_s = lc($days_s);

  my $days = 0x00;

  for my $expr (split /,/, $days_s) {
    my ($from_s, $to_s) = split /-/, $expr, 2;
    $to_s = $from_s unless defined $to_s;

    my ($from, $to) = ($DAYS{$from_s}, $DAYS{$to_s});
    croak "Unknown day: $from_s" unless defined $from;
    croak "Unknown day: $to_s" unless defined $to;

    if ($from <= $to) {
      for (my $i = 0; $i < 7; $i++) {
        my $day = 1 << $i;
        next if $day < $from && !($day & $from);
        last if $day > $to;
        $days |= $day;
      }
    }
    else {
      for (my $i = 0; $i < 7; $i++) {
        my $day = 1 << $i;
        next if $day < $from && $day > $to && !($day & ($from | $to));
        $days |= $day;
      }
    }
  }

  return $days;
}

sub _matches_days($$) {
  my ($now, $days_s) = @_;
  my $days = _parse_days($days_s);
  return ($days & (1 << $now->day_of_week_0)) != 0;
}

sub _matches_time($$) {
  my ($now, $time) = @_;

  return 1 if $time eq "*";

  my ($from, $to) = split /-/, $time, 2;
  my ($from_h, $from_m) = _parse_time($from);
  my ($to_h, $to_m)     = _parse_time($to);

  if ($now->hour < $from_h || ($now->hour == $from_h && $now->minute < $from_m)) {
    return 0;
  }
  elsif ($now->hour > $to_h || ($now->hour == $to_h && $now->minute > $to_m)) {
    return 0;
  }
  else {
    return 1;
  }
}

sub matches($$) {
  my $now = shift;
  my ($days, $time) = split /\s+/, shift, 2;

  return _matches_days($now, $days) && _matches_time($now, $time);
}

1;
# vim: et sw=2:
